<?php
namespace Teknisa\FIS\Modules\Acessorias\Controllers;

use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\File\File;
use Teknisa\FIS\Controllers\TeknisaCrud;
use Teknisa\Libs\Util\DateUtil;
use Teknisa\Libs\Util\Utilities;

class Fis75001ParametrosEfdReinfController extends TeknisaCrud 
{
    protected $Fis75001ParametrosEfdReinf;
    protected $dataSourceName = 'paramempreinf';
    protected $entityManager;
    protected $environment;
    protected $newCode;
    protected $fileUploader;
    protected $fileServerBaseUrl;
    protected $connection;
    
    public function __construct($crudService, $parameterBag, $environment, $entityManager, $newCode, $fileUploader, $fileServerBaseUrl)
    {
        parent::__construct($crudService, $parameterBag);
        $this->environment = $environment;
        $this->fileServerBaseUrl = $fileServerBaseUrl;
        $this->entityManager = $entityManager;
        $this->newCode = $newCode;
        $this->fileUploader              = $fileUploader;
        $this->connection = $entityManager->getConnection();
    }
    
    public function find(DTO\Request\Filter $request, DTO\Response $response)
    {
        try {
            $newResponse = $response;
            $this->dataSourceName = 'paramempreinf_classtribut_represlegal';
            parent::find($request, $response);
            $dataSets = $response->getDataSets();
            if(isset($dataSets[0])) {
                $rows = $dataSets[0]->getRows();
                if($rows) {
                    foreach($rows as &$row) {
                        $objB64 = $this->montaB64Certificado($row['DSDIRCERTIFICADO']);
                        $row['CERTIFICADONAME']   = $objB64['name'];
                        $row['DSDIRCERTIFICADO'] = [$objB64];
                    }
                } 
            }
            $dataSet = new DataSet('paramempreinf', $rows);
            $response->setDataSets(array($dataSet));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
        }
    } 
    
    public function save(DTO\Request $request, DTO\Response $response){
        try {
            $connection = $this->entityManager->getConnection();
            $connection->beginTransaction();
            $rows = $this->crudService->getRowsToSave($request, 'paramempreinf');
            
            $this->dataSourceName = 'paramempreinf';
            if(isset($rows[0]['DSDIRCERTIFICADO'][0])) {
                $file = $rows[0]['DSDIRCERTIFICADO'][0];
                $dsDirCert = '<'.$file['name'].'>'.$file['path'];
                $rows[0]['DSDIRCERTIFICADO'] = $dsDirCert;
            }
            else {
                $rows[0]['DSDIRCERTIFICADO'] = '';
            }
            if(isset($rows[0]['CDSENHACERTDIG']) && ($rows[0]['senhaAlterada'] === true || $rows[0]['__is_new'] === true)) {
                //$rows[0]['CDSENHACERTDIG'] = $this->encrypt($rows[0]['CDSENHACERTDIG']);
                $rows[0]['CDSENHACERTDIG'] = Utilities::zeedhiEncrypt($rows[0]['CDSENHACERTDIG']);
            }
            $this->crudService->save($this->dataSourceName, array($rows[0]));
            $this->entityManager->flush();

            if (!isset($rows[1]))  {
                $rows[1] = $rows[0];
                $rows[1]['DTFINHISTORICO'] = null;
                $rows[1]['DTINIHISTORICO'] = null;
            }

            $rows[1]['__is_new']  = true;
            $nrseqhist = $this->newCode->getCodeByTableName('NRSEQHISTH', $this->environment->getNrOrgTrab(), 1, 4);
            $rows[1]['NRSEQHIST'] = $nrseqhist;  
            $this->dataSourceName = 'paramempreinfh';

            $dtIniBackup = DateUtil::getDatadeString($rows[1]['DTINIVALIDADE'],
                $this->getFormat($rows[1]['DTINIVALIDADE']), true);

            $dtIni = DateUtil::getDatadeString($rows[0]['DTINIVALIDADE'],
                $this->getFormat($rows[0]['DTINIVALIDADE']), true);

            $dtFinBackup = $rows[1]['DTFINVALIDADE'] == null ? null :
                DateUtil::getDatadeString($rows[1]['DTFINVALIDADE'],
                    $this->getFormat($rows[1]['DTFINVALIDADE']), true);

            $dtFin = $rows[0]['DTFINVALIDADE'] == null ? null :
                DateUtil::getDatadeString($rows[0]['DTFINVALIDADE'],
                    $this->getFormat($rows[0]['DTFINVALIDADE']), true);

            // Compare dates is equals (3 to equal)
            if ((($dtFinBackup == null && $dtFin != null)
                  || ($dtFinBackup != null && $dtFin == null))
                || ( ($dtFinBackup != null && $dtFin != null)
                    && !DateUtil::comparaDatas($dtFinBackup, $dtFin, 3))) {
                $rows[1]['DTFINHISTORICO'] = $rows[1]['DTFINVALIDADE'];
                $rows[1]['DTFINVALIDADE'] = $rows[0]['DTFINVALIDADE'];    
            } else {
                $rows[1]['DTFINHISTORICO'] = $rows[0]['DTFINVALIDADE'];
            }

            // Compare dates is equals (3 to equal)
            if (!DateUtil::comparaDatas($dtIniBackup, $dtIni, 3)) {
                $rows[1]['DTINIHISTORICO'] = $rows[1]['DTINIVALIDADE'];
                $rows[1]['DTINIVALIDADE'] = $rows[0]['DTINIVALIDADE'];
            } else {
                $rows[1]['DTINIHISTORICO'] = $rows[0]['DTINIVALIDADE'];
            }

            $this->crudService->save($this->dataSourceName, array($this->adicionaCamposInsercao($rows[1])));
            $this->entityManager->flush();

            $connection->commit();
        } catch (\Exception $e) {
            $this->entityManager->rollback();
            $response->setError(new Error($e->getMessage(), 500));
        }
    } 
    
    public function update(DTO\Request $request, DTO\Response $response){
     
            $connection = $this->entityManager->getConnection();
            $connection->beginTransaction();
            $rows = $this->crudService->getRowsToSave($request, 'paramempreinf');
            
            $this->dataSourceName = 'paramempreinf';
            $objB64 = $this->salvaAnexoGrid($rows[0]['DSDIRCERTIFICADO']);
             
            if(isset($rows[0]['DSDIRCERTIFICADO'][0])) {
                $file = $rows[0]['DSDIRCERTIFICADO'][0];
                //var_dump($file);die;
                $dsDirCert = '<'.$file['name'].'>'.$file['webkitRelativePath'];
                $rows[0]['DSDIRCERTIFICADO'] = $dsDirCert;
            }
            else {
                $rows[0]['DSDIRCERTIFICADO'] = '';
            }
            
            
           
            $params = array(
                "CDEMPRESA"           => $rows['0']['CDEMPRESA'] ,
                "NRCLASSTRIBUT"       => $rows['0']['NRCLASSTRIBUT'],
                "IDESCRITURECD"       => $rows['0']['IDESCRITURECD'],
                "IDDESONERACAO"       => $rows['0']['IDDESONERACAO'],
                "IDACORDISENMULTA"    => $rows['0']['IDACORDISENMULTA'],
                "IDSITUPJ"            => $rows['0']['IDSITUPJ'],
                "CDREPRES"            => $rows['0']['CDREPRES'],
                "DTINIVALIDADE"       => $rows['0']['DTINIVALIDADE'],
                "DTFINVALIDADE"       => $rows['0']['DTFINVALIDADE'],
                "IDINDCPRB"           => $rows['0']['IDINDCPRB'],
                "CDVERSAO"            => $rows['0']['CDVERSAO'], 
                "IDAMBIENTE"          => $rows['0']['IDAMBIENTE'],
                "CDSENHACERTDIG"      => $rows['0']['CDSENHACERTDIG'],
                "IDLOCALSERVIDOR"     => $rows['0']['IDLOCALSERVIDOR'],
                "DSDIRCERTIFICADO"    => $rows['0']['DSDIRCERTIFICADO'],
                "NRORG"               => $rows['0']['NRORG'],
                "IDATIVO"             => $rows['0']['IDATIVO'],
                "DTINCLUSAO"          => $rows['0']['DTINCLUSAO'],
                "DTULTATU"            => $rows['0']['DTULTATU'],
                "NRORGINCLUSAO"       => $rows['0']['NRORGINCLUSAO'],
                "CDOPERINCLUSAO"      => $rows['0']['CDOPERINCLUSAO'],
                "NRORGULTATU"         => $rows['0']['NRORGULTATU'],
                "CDOPERULTATU"        => $rows['0']['CDOPERULTATU']
              //  "NRCERTIFICDIG"       => $rows['0']['NRCERTIFICDIG'],
              //  "CDATIVIDADE"         => $rows['0']['CDATIVIDADE'],
              //  "VRALIQCPRB"          => $rows['0']['VRALIQCPRB']
            );
            $sql = "UPDATE PARAMEMPREINF 
                        SET
                            NRCLASSTRIBUT       = :NRCLASSTRIBUT,
                            IDESCRITURECD       = :IDESCRITURECD,
                            IDDESONERACAO       = :IDDESONERACAO,
                            IDACORDISENMULTA    = :IDACORDISENMULTA,
                            IDSITUPJ            = :IDSITUPJ,
                            CDREPRES            = :CDREPRES,
                            DTINIVALIDADE       = :DTINIVALIDADE,
                            DTFINVALIDADE       = :DTFINVALIDADE,
                            IDINDCPRB           = :IDINDCPRB,
                            CDVERSAO            = :CDVERSAO, 
                            IDAMBIENTE          = :IDAMBIENTE,
                            CDSENHACERTDIG      = :CDSENHACERTDIG,
                            IDLOCALSERVIDOR     = :IDLOCALSERVIDOR,
                            DSDIRCERTIFICADO    = :DSDIRCERTIFICADO,
                            NRORG               = :NRORG,
                            IDATIVO             = :IDATIVO,
                            DTINCLUSAO          = :DTINCLUSAO,
                            DTULTATU            = :DTULTATU,
                            NRORGINCLUSAO       = :NRORGINCLUSAO,
                            CDOPERINCLUSAO      = :CDOPERINCLUSAO,
                            NRORGULTATU         = :NRORGULTATU,
                            CDOPERULTATU        = :CDOPERULTATU
                            WHERE
                            CDEMPRESA          = :CDEMPRESA 
                            ";
       // Prepara o statement da query para execução.
         $std = $this->connection->prepare($sql);
           //var_dump($params);die;
        $std->execute($params);
        
         $connection->commit();
        $this->entityManager->flush();

    }
    
    private function adicionaCamposInsercao($row) {
        $row['DTINCLUSAO'] = $this->environment->getCurrentTime()->format("d/m/Y H:i:s");
        $row['NRORGINCLUSAO'] = $this->environment->getNrOrgTrab();
        $row['CDOPERINCLUSAO'] = $this->environment->getCdOperador();
        return $row;
    }
    
    private function montaB64Certificado($dsDirCertificado) {
        if($dsDirCertificado) {
            $fileBase = explode('>', $dsDirCertificado);
            $fileName = $fileBase[0];
            $fileName = str_replace('<', '', $fileName);
            $fileLink = $fileBase[1];
            $file = file_get_contents($this->fileServerBaseUrl.$fileLink);
            $b64File = base64_encode($file);

            $tmpFile = sys_get_temp_dir().DIRECTORY_SEPARATOR.'certificado'.uniqid().'.pfx';
            file_put_contents($tmpFile, $file);
            $file = new File($tmpFile);
            $certificado = file_get_contents($tmpFile);
            $certPfx = base64_encode($certificado);
    
            $certPfx = 'data:application/x-pkcs12;base64,'.$certPfx;
            
            $lastModify = filectime($tmpFile);
    
            $certFileB64 = array(
                "name" => $fileName, 
                "lastModified" => $lastModify,
                "lastModifiedDate" => date("D,d M Y H:i:s", $lastModify)." GMT",
                "webkitRelativePath" => '',
                "type" => $file->getMimeType(),
                "size"  => filesize($tmpFile),
                "b64File" => $certPfx);
            
            return $certFileB64;
        }
    }
    
    private function encrypt($text, $salt = 'teknisa') {
        return trim(
            base64_encode(
                mcrypt_encrypt(
                    MCRYPT_RIJNDAEL_256,
                    str_pad($salt, 32, "\0"),
                    $text,
                    MCRYPT_MODE_ECB,
                    mcrypt_create_iv(mcrypt_get_iv_size(MCRYPT_RIJNDAEL_256, MCRYPT_MODE_ECB), MCRYPT_RAND)
                )
            )
        );
    }
    
    public function decrypt($text, $salt = 'teknisa'){
        return trim(
            mcrypt_decrypt(
                MCRYPT_RIJNDAEL_256,
                str_pad($salt, 32, "\0"),
                base64_decode($text),
                MCRYPT_MODE_ECB,
                mcrypt_create_iv(mcrypt_get_iv_size(MCRYPT_RIJNDAEL_256, MCRYPT_MODE_ECB), MCRYPT_RAND)
            )
        );
    }

    private function getFormat($stringDate)
    {
        //10 is brazilian format, more is date hour brazilian format
        if (strlen($stringDate) === 10) {
            return DateUtil::FORMATO_BRASILEIRO;
        } else {
            return DateUtil::FORMATO_BRASILEIRO_DATAHORA;
        }
    }
    
     public function saveXmlInFileUplouder($arquivo) {
       
        if (isset($arquivo)) {
            
            $file = array(
                "name" => time().'.'.explode('/', $arquivo["type"])[1], 
                "lastModified" => $arquivo["lastModified"],
                "lastModifiedDate" => $arquivo["lastModifiedDate"],
                "webkitRelativePath" => $arquivo["webkitRelativePath"],
                "type" => $arquivo["type"],
                "size"  => $arquivo["size"], 
                "b64File" => $arquivo["b64File"]);
               
            $row['DSDIRCERTIFICADO']    = array();
            $row['DSDIRCERTIFICADO'][0] = $file;
              
            $this->fileUploader->uploadFilesInRow($row, 'DSDIRCERTIFICADO', 'DSDIRCERTIFICADO');
          //  var_dump($row);die;
            return $row['DSDIRCERTIFICADO'][0]['path'];
            
            /*
            
            $row = [
                'FILE' => [
                    [
                        "name" => time().'.'.explode('/', $arquivo["type"])[1], 
                        "type" => $arquivo["type"], 
                        "size" => $arquivo["size"], 
                        "path" => '', 
                        "b64File" => $arquivo["b64File"]
                    ]
                ],
                'FOLDER' => 'ANEXOLANCBOLREC'
            ];
            
            return $this->fileUploader->uploadFile($row)[0]->getRows()[0]['FILE'][0]['PATH'];
            */
            
        } else {
            return false;
        }
    }
    
    public function salvaAnexoGrid($row){
       
        try {
            $connection = $this->entityManager->getConnection();
            $connection->beginTransaction();
            
            //$row['DSDIRCERTIFICADO'] = $this->geraNovoCodigo($row['NRORG'], 'DSDIRCERTIFICADO');
            //Salva o arquivo sempre como txt independente do seu formato
             
            $row['DSINFOANEXO'] = $row[0]["lastModified"] . '_' . $row[0]["lastModifiedDate"] . '_' . 
                                $row[0]["webkitRelativePath"]. '_' . $row[0]["type"] . '_' . 
                                $row[0]["size"] . '_' . $row[0]["name"];
            
            $row['NMPATHANEXO'] = $this->saveXmlInFileUplouder($row[0]);
        
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function geraNovoCodigo(
        $nrorg, $cdcontador, $intervalo = 1, $tamanho = 10
    ){
        try {
            // Gera Novo Código Boleto
            $novocodigo = $this->newCode->getCodeByTableName($cdcontador, $nrorg, $intervalo, $tamanho);
            return $novocodigo;
        } catch (\Exception $e) {
            throw $e;
        }
    }
}