<?php
namespace Teknisa\FIS\Modules\Acessorias\Services;

use Teknisa\FIS\Modules\Acessorias\Factories\Fis91000ObrigacoesAcessoriasFactory;
use Teknisa\Libs\Util\DateUtil;
use Teknisa\Libs\Util\NewCode;

class Fis91000ObrigacoesAcessoriasService {

    protected $newCode;
    protected $obrigacoesFactory;

    public function __construct(NewCode $newCode, Fis91000ObrigacoesAcessoriasFactory $obrigacoesFactory) {
        $this->newCode = $newCode;
        $this->obrigacoesFactory = $obrigacoesFactory;
        set_time_limit(0);
    }

    /*
     * Função para criar um arquivo a ser exportado.
     */
    public function createFileExportacao($fileName, $data, $columnName = "DSREGARQMAG") {
        // Verifica se algum dado foi retornado, lançando uma exceção caso contrário.
        if(empty($data)) {
            throw new \Exception("Não há dados a serem exibidos.");
        }

        // Verifica se há uma extensão já informada.
        if(empty(pathinfo($fileName, PATHINFO_EXTENSION)) || substr($fileName, -1) == '.') {
            // Se o nome do arquivo for terminado em '.', termina a extensão '.txt'.
            if (substr($fileName, -1) == '.') {
                $fileName .= "txt";
            }
            // Senão, adiciona a extensão '.txt' ao nome do arquivo.
            else {
                $fileName .= ".txt";
            }
        }

        // Cria ou sobrepõe um arquivo com o nome dado.
        try {
            $file = fopen(sys_get_temp_dir() . DIRECTORY_SEPARATOR . $fileName, "w");
        } catch (\Exception $e) {
            // Se houver algum erro ao tentar abrir o arquivo, lança uma exceção.
            throw new \Exception("Ocorreu um erro ao criar o arquivo para exportação.");
        }
        // Se houver algum erro após tentar abrir o arquivo, lança uma exceção.
        if (!$file) {
            throw new \Exception("Ocorreu um erro ao criar o arquivo para exportação.");
        }

        // Inicializa a variável com o texto a ser impresso no arquivo.
        $text = "";
        // Percorre os dados, concatenando na variável a ser impressa.
        foreach ($data as $dataRow) {
            $text .= $dataRow["DSREGARQMAG"] . "\r\n";
        }

        // Imprime o texto no arquivo.
        $text = iconv("UTF-8", "WINDOWS-1252", $text);
         
        fprintf($file, "%s", $text);

        // Fecha o arquivo após as alterações.
        fclose($file);
        
        // Retorna o nome do arquivo com suas possíveis modificações.
        return $fileName;
    }

    /*
     * Função para retornar um array associativo contendo o primeiro e o último
     * dia do mês referência.
     */
    public function getFirstAndLastDays($stringDate) {
        // Cria o objeto DateTime para manipulação no utilitário.
        $referenceDate = DateUtil::getDataDeString($stringDate);

        // Retorna um array associativo contendo as datas.
        return array(
            "DTINICIAL" => DateUtil::getPrimeiroDiaDoMes($referenceDate)->format(DateUtil::FORMATO_BRASILEIRO),
            "DTFINAL" => DateUtil::getUltimoDiaDoMes($referenceDate)->format(DateUtil::FORMATO_BRASILEIRO)
        );
    }
    
    /*
     * Função para retornar um array associativo contendo o primeiro e o último
     * dia do mês referência.
     */
    public function getFirstAndLastDaysGIAMTO($stringDate) {
        // Cria o objeto DateTime para manipulação no utilitário.
        $referenceDate = DateUtil::getDataDeString($stringDate);

        // Retorna um array associativo contendo as datas.
        return array(
            "DTINICIO" => DateUtil::getPrimeiroDiaDoMes($referenceDate)->format(DateUtil::FORMATO_BRASILEIRO),
            "DTFIM" => DateUtil::getUltimoDiaDoMes($referenceDate)->format(DateUtil::FORMATO_BRASILEIRO)
        );
    }

    /*
     * Função para retornar no próximo sequencial dos arquivos de exporação.
     */
    public function getNextNrseqarqmag($nrorg) {
        return $this->newCode->getCodeByTableName('ARQMAGNETICO', $nrorg);
    }

    /*
     * Função para buscar os "Parâmetros de Fiscal" da organização.
     */
    public function getParamfiscal($nrorg) {
        $params = array(
            "NRORG" => $nrorg
        );

        return $this->obrigacoesFactory->getParamfiscal($params);
    }

    /*
     * Função que verifica o mapa de apuração conforme parametrização fiscal.
     */
    public function mapaApuracIsValid($row, $impostos = array()) {
        // Busca os "Parâmetros de Fiscal" da organização.
        $paramfiscal = $this->getParamfiscal($row["NRORG"]);
        $isValid = true;

        // Verifica se foi retornado algum resultado.
        if ($paramfiscal == null) {
            throw new \Exception("Parâmetros de Fiscal não cadastrados.");
        }

        // Caso o campo "Gerar Declarações Acessórias com o Mapa Bloqueado" dos
        // parâmetros esteja marcado, é verificado se todos os mapas de apuração
        // estão bloqueados antes da exportação.
        if ($paramfiscal["IDGERDECMPBLOQ"] == 'S') {
            $params = array(
                "NRORG" => $row["NRORG"],
                "CDEMPRESA" => $row["CDEMPRESA"],
                "CDINSCESTA" => empty($row["CDINSCESTA"]) ? null : $row["CDINSCESTA"],
                "DTINICIAL" => $row["DTINICIAL"],
                "DTFINAL" => $row["DTFINAL"],
                "IMPOSTOS" => implode($impostos, ',')
            );
            $mapaapurac = $this->obrigacoesFactory->getMapaapurac($params);

            // Verifica se algum dado foi retornado, indicando que o mapa
            // está inválido.
            if ($mapaapurac != null) {
                $isValid = false;
            }
        }

        return $isValid;
    }
}