<?php

namespace Teknisa\FIS\Modules\Guias\Factories;

use Doctrine\ORM\EntityManager;
use Zeedhi\Framework\DB\StoredProcedure\Param;
use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;
use Teknisa\Model\Financeiro\Entities\Movfluxo;
use Teknisa\Model\Fiscal\Entities\Notafiscal;
use Teknisa\Model\Financeiro\Entities\Boletpagar;
use Teknisa\Model\Financeiro\Entities\Contapagar;
use Teknisa\Model\Financeiro\Entities\Ratctpagar;
use Teknisa\Libs\Util\DateUtil;


class Fis00200DocumentoDeArrecadacaoFactory{
    protected $connection;
    protected $entityManager;

    public function __construct(EntityManager $entityManager) {
        $this->connection = $entityManager->getConnection();
        $this->entityManager = $entityManager;
    }

    public function getParamguiapag($params) {
        
        //Caso nao exista imposto(Simples Nacional) a query será diferente
        if($params['CDIMPOSTO']){
            $sql = "SELECT P.*, F.NMFILIAL, R.NMRATEIO FROM PARAMGUIAPAG P , FILIAL F, PADRATEIO R WHERE P.CDEMPRESA = :CDEMPRESA AND P.CDIMPOSTO = :CDIMPOSTO AND P.IDTIPOEMISSAO = :IDTIPOEMISSAO AND P.CDFILIAL = F.CDFILIAL AND P.CDRATEIO = R.CDRATEIO(+)";    
        }else{
            unset($params['CDIMPOSTO']);
            $sql = "SELECT P.*, F.NMFILIAL, R.NMRATEIO FROM PARAMGUIAPAG P , FILIAL F, PADRATEIO R WHERE P.CDEMPRESA = :CDEMPRESA AND P.CDIMPOSTO IS NULL AND P.IDTIPOEMISSAO = :IDTIPOEMISSAO AND P.CDFILIAL = F.CDFILIAL AND P.CDRATEIO = R.CDRATEIO(+)";
        }
        
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetch();
    }
    
    public function getParamtrans($params) {
        
        $sql = "SELECT * FROM PARAMTRANS WHERE CDEMPRESA = :CDEMPRESA AND CDINSCESTA = :CDINSCESTA";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetch();
    }
    
    public function getImpostoNfe($params) {
        $sql = "SELECT CDIMPOSTONFE, IDORGARRECADNFE, IDFORMAPURNFE, IDIMPORECUPNFE FROM IMPOSTONFE I WHERE I.NRORG = :NRORG AND I.CDIMPOSTONFE = :CDIMPOSTONFE";
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetch();
    }

    public function getImpostoRelac($params) {
        $sql = "SELECT CDIMPOSTO, IDORGARRECAD, IDFORMAPUR, IDIMPORECUP FROM IMPOSTO I WHERE I.NRORG = :NRORG AND I.CDIMPOSTONFE = :CDIMPOSTONFE";
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
    
    public function getIdbloqmapapu($params) {
        $sql = "SELECT IDBLOQMAPAPU FROM MAPAAPURAC WHERE NRSEQUMAPAPU = :NRSEQUMAPAPU UNION ALL SELECT IDBLOQMAPAPU FROM MAPAAPURACSN WHERE NRSEQUMAPAPU = :NRSEQUMAPAPU AND NRORG = :NRORG";

        $std = $this->connection->prepare($sql);
        $std->execute($params);

        return $std->fetch()["IDBLOQMAPAPU"];
    }
    
    public function getDocarrecadpag($row) {
        $sql = "SELECT * FROM DOCARRECADPAG";
        
        $idtipoguia     = $row['IDTIPOGUIA'];
        $nrsequmapapu   = $row['NRSEQUMAPAPU'];
        $nrseqguiacompl = $row['NRSEQGUIACOMPL'];
        $cdfilial       = $row['CDFILIAL'];
        $nrlanctonf     = $row['NRLANCTONF'];
        $cdimposto      = $row['CDIMPOSTO'];
        
        switch ($idtipoguia) {
            //Emissão através de uma Guia Complementar
            case 'G':
                $params = array(
                    "NRSEQUMAPAPU"   => $nrsequmapapu,
                    "NRSEQGUIACOMPL" => $nrseqguiacompl
                );
                $sql = $sql.' WHERE NRSEQUMAPAPU = :NRSEQUMAPAPU AND NRSEQGUIACOMPL = :NRSEQGUIACOMPL';
                break;
            //Emissão através de uma Mapa
            case 'M':
                $params = array(
                    "NRSEQUMAPAPU" => $nrsequmapapu
                );
                $sql = $sql.' WHERE NRSEQUMAPAPU = :NRSEQUMAPAPU AND NRSEQGUIACOMPL IS NULL';
                break;
            //Emissão através de uma Mapa do Simples Nacional
             case 'S':
                $params = array(
                    "NRSEQUMAPAPU" => $nrsequmapapu
                );
                $sql = $sql.' WHERE NRSEQUMAPAPUSN = :NRSEQUMAPAPU AND NRSEQGUIACOMPL IS NULL';
                break;
            //Emissão através de uma Nota Fiscal (Documento) - Obs.: validaMapaBloqueado() também usa esta verificação
            case 'I': //Imposto Normal
            case 'O': //Difal Não Contribuinte Origem
            case 'D': //Difal Não Contribuinte Destino
            case 'F': //Fundo de Combate a Pobreza Não Contribuinte
            case 'P': //FECP
            case 'A': //Diferencial de Alíquota
                $params = array(
                    "CDFILIAL"   => $cdfilial,
                    "NRLANCTONF" => $nrlanctonf,
                    "CDIMPOSTO"  => $cdimposto,
                    "IDTIPOGUIA" => $idtipoguia
                );
                $sql = $sql.' WHERE CDFILIAL = :CDFILIAL AND NRLANCTONF = :NRLANCTONF AND CDIMPOSTO = :CDIMPOSTO AND IDTIPOGUIA = :IDTIPOGUIA';
                break;
            //todo tem as outyras guias criadas aqui
        }
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetch();
    }
    
    public function insertDocarrecadpag($params) {
        $sql = " INSERT INTO DOCARRECADPAG (
                    NRSEQDOCARRECAD,
                    NRSEQUMAPAPUSN,
                    NRSEQUMAPAPU,
                    NRSEQGUIAAPURNC,
                    NRSEQMAPAPURESTNC,
                    NRSEQGUIACOMPL,
                    CDFILIAL,
                    CDFILIALPAG,
                    NRLANCTONF,
                    NRLANCTONFPAG,
                    CDIMPOSTO,
                    DTVENCIMENTO,
                    CDRECEITA,
                    CDNATUREZA,
                    CDDETRECEITA,
                    IDTIPOGUIA,
                    NRORG,
                    VRMULTA,
                    VRJUROS,
                    VRPRINCIPAL,
                    DTLIMITEPAG,
                    DSINFOCOMPL
                )
                VALUES (
                    :NRSEQDOCARRECAD,
                    :NRSEQUMAPAPUSN,
                    :NRSEQUMAPAPU,
                    :NRSEQGUIAAPURNC,
                    :NRSEQMAPAPURESTNC,
                    :NRSEQGUIACOMPL,
                    :CDFILIAL,
                    :CDFILIALPAG,
                    :NRLANCTONF,
                    :NRLANCTONFPAG,
                    :CDIMPOSTO,
                    :DTVENCIMENTO,
                    :CDRECEITA,
                    :CDNATUREZA,
                    :CDDETRECEITA,
                    :IDTIPOGUIA,
                    :NRORG,
                    :VRMULTA,
                    :VRJUROS,
                    :VRPRINCIPAL,
                    :DTLIMITEPAG,
                    :NMMAPAAPUR 
                )";
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    
        
    }
    
    public function updateDocarrecadpagNewLote($nrseqlote) {
        $params = array(
            "NRSEQLOTE"    => $nrseqlote,
        );
        
        $sql = "UPDATE DOCARRECADPAG SET NRSEQLOTE = NULL, NRNOSSONUMERO = NULL, NRCODIGOBARRA = NULL, NRLINHADIGI = NULL, IDTRANSMISSAO = 'N' WHERE NRSEQLOTE = :NRSEQLOTE";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
    
    public function updateDocarrecadpag($cdnatureza, $cdreceita, $cddetreceita, $dtvencimento, $cdfilialpag, $nrlanctonfpag, $nrseqdocarrecad, $vrmulta, $vrjuros, $vrprincipal, $dtlimitepag, $nrseqguiacompl, $nmmapaapur, $idtipoguia = null, $nrsequmapapu = null) {
        $params = array(
            "NRSEQDOCARRECAD" => $nrseqdocarrecad,
            "DTVENCIMENTO"    => $dtvencimento,
            "CDFILIALPAG"     => $cdfilialpag,
            "NRLANCTONFPAG"   => $nrlanctonfpag,
            "CDNATUREZA"      => $cdnatureza,
            "CDRECEITA"       => $cdreceita,
            "CDDETRECEITA"    => $cddetreceita,
            "VRMULTA"         => $vrmulta,
            "VRJUROS"         => $vrjuros,
            "VRPRINCIPAL"     => $vrprincipal,
            "IDTIPOGUIA"      => $idtipoguia, 
            "DTLIMITEPAG"     => $dtlimitepag,
            "NRSEQGUIACOMPL"  => $nrseqguiacompl,
            "NMMAPAAPUR"      => $nmmapaapur
        );

        $sql = "UPDATE DOCARRECADPAG SET CDNATUREZA = :CDNATUREZA, CDRECEITA = :CDRECEITA, CDDETRECEITA = :CDDETRECEITA, DTVENCIMENTO = :DTVENCIMENTO, CDFILIALPAG = :CDFILIALPAG, NRLANCTONFPAG = :NRLANCTONFPAG, VRMULTA = :VRMULTA, VRJUROS = :VRJUROS, VRPRINCIPAL = :VRPRINCIPAL, DTLIMITEPAG = :DTLIMITEPAG, NRSEQGUIACOMPL = :NRSEQGUIACOMPL, DSINFOCOMPL = :NMMAPAAPUR WHERE NRSEQDOCARRECAD = :NRSEQDOCARRECAD AND IDTIPOGUIA = :IDTIPOGUIA";
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        //Atualiza tabelas delphi
        if($nrlanctonfpag != null) {
            if($idtipoguia == 'M') {
                $params = array( "CDFILIALPAG"   => $cdfilialpag,
                                 "NRLANCTONFPAG" => $nrlanctonfpag,
                                 "NRSEQUMAPAPU"  => $nrsequmapapu );
                $this->updateMapaapurac($params);
            } elseif($idtipoguia == 'G') {
                $params = array( "CDFILIALPAG"   => $cdfilialpag,
                                 "NRLANCTONFPAG" => $nrlanctonfpag,
                                 "DTVENCIMENTO"  => $dtvencimento,
                                 "NRSEQUMAPAPU"  => $nrsequmapapu );
                $this->insertRelacmapanf($params); //Se não houver registro este o insere
            }
        } else {
            if($idtipoguia == 'M') {
                $params = array( "CDFILIALPAG"   => "",
                                 "NRLANCTONFPAG" => "",
                                 "NRSEQUMAPAPU"  => $nrsequmapapu );
                $this->updateMapaapurac($params);
            } elseif($idtipoguia == 'G') {
                $params = array( "CDFILIALPAG"   => $cdfilialpag,
                                 "NRLANCTONFPAG" => $nrlanctonfpag,
                                 "NRSEQUMAPAPU"  => $nrsequmapapu );
                $this->deleteRelacmapanf($params);
            }
        }
        /*$std = $this->connection->prepare($sql);
        $std->execute($params);*/
    }
    
    /*Apenas insere no RELACMAPANF se não existir registro correspondênte*/
    public function insertRelacmapanf($params) {
        $sql = "SELECT * FROM RELACMAPANF WHERE CDFILIAL = :CDFILIAL AND NRLANCTONF = :NRLANCTONF AND NRSEQUMAPAPU = :NRSEQUMAPAPU";
        $std = $this->connection->prepare($sql);
        $std->execute(['CDFILIAL'    => $params['CDFILIALPAG'],
                       'NRLANCTONF'  => $params['NRLANCTONFPAG'],
                       'NRSEQUMAPAPU'=> $params['NRSEQUMAPAPU']]);
        $res = $std->fetch();
        if(!$res) {
            $sql = "INSERT INTO RELACMAPANF (CDFILIAL, NRLANCTONF, DTVENCMAPANF, IDBLOQMAPANF, NRSEQUMAPAPU)
                    VALUES (:CDFILIALPAG, :NRLANCTONFPAG, :DTVENCIMENTO, 'S', :NRSEQUMAPAPU)";
            $std = $this->connection->prepare($sql);
            $std->execute($params);
        }
    }
    
    /*Update do MAPAAPURAC*/
    public function updateMapaapurac($params) {
        $sql = "UPDATE MAPAAPURAC SET CDFILIAL = :CDFILIALPAG, NRLANCTONF = :NRLANCTONFPAG WHERE NRSEQUMAPAPU = :NRSEQUMAPAPU";
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
    
    /*Delete do RELACMAPANF*/
    public function deleteRelacmapanf($params) {
        $sql = "DELETE FROM RELACMAPANF WHERE CDFILIAL = :CDFILIALPAG AND NRLANCTONF = :NRLANCTONFPAG AND NRSEQUMAPAPU = :NRSEQUMAPAPU";
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
    
    public function insertParamguiapag($params) {
        $sql = " INSERT INTO PARAMGUIAPAG (
                    NRSEQDADOS, CDFILIAL, CDRATEIO, CDIMPOSTO, CDEMPRESA, IDTIPOEMISSAO, NRORG
                )
                VALUES
                    (:NRSEQDADOS, :CDFILIAL, :CDRATEIO, :CDIMPOSTO, :CDEMPRESA,:IDTIPOEMISSAO, :NRORG)";
        $std = $this->connection->prepare($sql);
        $std->execute($params);

    }
    
    public function updateParamguiapag($cdfilial, $cdrateio, $nrseqdados) {
        $params = array(
            "NRSEQDADOS" => $nrseqdados,
            "CDRATEIO"   => $cdrateio,
            "CDFILIAL"  => $cdfilial
        );
        $sql = " UPDATE PARAMGUIAPAG SET CDFILIAL =  :CDFILIAL, CDRATEIO = :CDRATEIO WHERE NRSEQDADOS =:NRSEQDADOS";
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
    
    public function getParametrosEmissaoGuia($cdimposto) {
        $sql = "SELECT P.*, TC.CDCLASSFINA FROM PARAMIMPOSGUIA P, TIPOCTPAGAR TC WHERE CDIMPOSTO = :CDIMPOSTO AND P.CDTPCTPAGAR = TC.CDTPCTPAGAR AND P.NRORG = TC.NRORG";
       
        $params = array(
            "CDIMPOSTO"  => $cdimposto
        );
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetch();
    }
    
    public function getParametrosFiscal() {
        $sql = "SELECT * FROM PARAMFISCAL";

        $std = $this->connection->prepare($sql);
        $std->execute();
        return $std->fetch();
    }
    
    public function getCentrodeCusto($cdfilial) {
        $sql = "SELECT NVL(FI.CDCENTCUST, E.CDCENTCUST) AS CDCENTCUST
                FROM FILIAL FI,
                     EMPRESA E
                WHERE FI.CDFILIAL = :P_CDFILIAL
                AND FI.CDEMPRESA  = E.CDEMPRESA";
       
        $params = array(
            "P_CDFILIAL"  => $cdfilial
        );
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetch();
    }
    
    public function getBeneficiario($cdempresa, $inscricao, $cdimpostonfe, $idorgarrecadnfe) {
        $sql = "
            SELECT
              CASE
                WHEN :IDORGARRECADNFE = 'F' THEN NVL( MAX( BENEF2 ), MAX( BENEF1 ) )
                ELSE NVL( MAX( BENEF1 ), MAX( BENEF2 ) )
              END AS CDFORNECED
            FROM (
                SELECT P.CDFORNECED BENEF1, '' as BENEF2
                FROM PARAMBENEFIMP P ,  EMPRESA E
                WHERE  E.CDEMPRESA = :CDEMPRESA
                  AND P.CDPAIS = E.CDPAIS
                  AND :IDORGARRECADNFE = 'F'
                  AND P.IDTPPARAMBENEF ='F'
              UNION ALL
                SELECT P.CDFORNECED BENEF1, '' as BENEF2
                FROM PARAMBENEFIMP P , INSCRESTAD INSC
                WHERE INSC.CDINSCESTA = :INSCRICAO
                  AND INSC.CDEMPRESA = :CDEMPRESA
                  AND P.CDPAIS = INSC.CDPAIS
                  AND P.SGESTADO = INSC.SGESTADO
                  AND :IDORGARRECADNFE = 'E'
                  AND P.IDTPPARAMBENEF='E'
              UNION ALL 
                SELECT  P.CDFORNECED BENEF1, '' as BENEF2
                FROM PARAMBENEFIMP P , INSCRMUNICIP INSC
                WHERE  INSC.CDINSCMUNI = :INSCRICAO
                  AND INSC.CDEMPRESA = :CDEMPRESA
                  AND P.CDPAIS = INSC.CDPAIS
                  AND P.SGESTADO = INSC.SGESTADO
                  AND P.CDMUNICIPIO = INSC.CDMUNICIPIO
                  AND :IDORGARRECADNFE = 'M'
                  AND P.IDTPPARAMBENEF='M'
              UNION ALL
                SELECT '' AS BENEF1, P.CDFORNECED AS BENEF2
                FROM PARAMIMPOSGUIA P
                WHERE P.CDIMPOSTO = :CDIMPOSTONFE
            )
        ";
        $params = array(
            "CDEMPRESA"    => $cdempresa,
            "INSCRICAO"    => $inscricao,
            "CDIMPOSTONFE" => $cdimpostonfe,
            "IDORGARRECADNFE" => $idorgarrecadnfe
        );
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetch();
    }
    
    public function getInscricaoFilial($cdempresa) {
        $sql = "SELECT CDINSCESTA, CDINSCMUNI
                FROM FILIAL 
                WHERE CDEMPRESA  = :CDEMPRESA";
       
        $params = array(
            "CDEMPRESA"  => $cdempresa
        );
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetch();
    }
    
    public function getItensRateio ($cdrateio){
        $sql = " SELECT CDFILIAL AS CDFILICCUST, CDCENTCUST, (VRRATEIO/100) AS VRPERATTPCTP
                   FROM ITEMRATEIO
                  WHERE CDRATEIO = :CDRATEIO";
       
        $params = array(
            "CDRATEIO"  => $cdrateio
        );
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
       
    }
    
    public function atualizaDiferencaRateio ($cdfiratctpag, $cdcentcust,  $cdfilial, $nrlanctonf, $diferenca){
        $sql = "UPDATE RATCTPAGAR
                SET VRRATCCCTPAG = VRRATCCCTPAG + :DIFERENCA WHERE 
                CDFIRATCTPAG   = :CDFIRATCTPAG
                AND CDCENTCUST = :CDCENTCUST 
                AND CDFILIAL   = :CDFILIAL
                AND NRLANCTONF = :NRLANCTONF";
       
        $params = array(
            "CDFIRATCTPAG"  => $cdfiratctpag,
            "CDCENTCUST"    => $cdcentcust,
            "CDFILIAL"      => $cdfilial,
            "NRLANCTONF"    => $nrlanctonf,
            "DIFERENCA"     => $diferenca
        );
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);

    }
    
    public function getBoleto($cdfilial, $nrlanctonf) {
        $sql = "SELECT C.*, B.*
                FROM CONTAPAGAR C,
                     BOLETPAGAR B
                WHERE C.CDFILIAL  = :CDFILIAL
                AND C.NRLANCTONF  = :NRLANCTONF
                AND C.CDFILIBPAG  = B.CDFILIAL
                AND C.NRLANBOLPAG = B.NRLANBOLPAG";
        
        $params = array(
            "CDFILIAL"  => $cdfilial,
            "NRLANCTONF"=>$nrlanctonf
        );
      
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetch();
    }
    
    public function getComplementInfo($sgestado, $nrlanctonf, $cdinscesta, $idtpdoc, $tprespons, $cdempresa) {
        $sql = "SELECT 
                    N.NRNOTAFISC AS NUMERO_NF,
                    IE.NRINSJURIEST AS CNPJ,
                    NVL(N.CDSERIE, N.CDSERIEENTR ) AS SERIE,
                    MODELO_DOCUMENTO_GNRE(N.CDMODDOCFISC, DECODE(:SGESTADO , 'RJ', '2', '1')) AS TIPO_NF,
                    '1' AS TIPO_IMPORT,
                    11 AS DOC_ORIGEM,
                    DECODE(N.IDENTRSAIDOP,'S',N.DTEMISSAO,N.DTENTRSAID) AS DATA_FATO_GER,
                    CODIGO_PRODUTO_GNRE_RJ(:P_IDTPDOC, :P_TPRESPONS, :P_CDEMPRESA) AS PRODUTO,
                    N.DTEMISSAO
                    FROM
                    NOTAFISCAL N, INSCRESTAD IE 
                    WHERE 
                    N.NRLANCTONF = :NRLANCTONF AND
                    N.CDINSCESTA = :CDINSCESTA AND
                    N.CDINSCESTA = IE.CDINSCESTA";
        
        $params = array(
            "SGESTADO"  => $sgestado,
            "P_IDTPDOC"=>$idtpdoc,
            "P_TPRESPONS"=>$tprespons,
            "P_CDEMPRESA"=>$cdempresa,
            "NRLANCTONF"=>$nrlanctonf,
            "CDINSCESTA"=>$cdinscesta,
        );
      
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetch();
    }
    
    public function atualizaBoleto($cdfilial, $nrlanbolpag, $dslinhbarpag, $cdbarrapag) {
        $params = array(
            "CDFILIAL"    => $cdfilial,
            "NRLANBOLPAG"   => $nrlanbolpag,
            "DSLINHBARPAG"  => $dslinhbarpag,
            "CDBARRAPAG"    => $cdbarrapag
        );
        $sql = " UPDATE  BOLETPAGAR SET DSLINHBARPAG = :DSLINHBARPAG, CDBARRAPAG= :CDBARRAPAG WHERE NRLANBOLPAG = :NRLANBOLPAG AND CDFILIAL = :CDFILIAL";
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
    
    public function getEmpresa($cdempresa) {
        $sql = "SELECT P.NMPAIS, S.NMESTADO, M.NMMUNICIPIO, B.NMBAIRRO, L.DSLOGRADOURO, CO.NMCONTADOR, CE.NMCENTCUST, CN.DSCNAE, FP.DSFPAS, NA.DSNATUJURI, E.* FROM EMPRESA E, PAIS P, ESTADO S, MUNICIPIO M, BAIRRO B, LOGRADOURO L, CONTADOR CO, CENTCUST CE, CNAEFISCAL CN, FPAS FP, NATUREZAJURI NA WHERE E.CDCONTADOR2 = CO.CDCONTADOR2(+) AND E.CDCENTCUST = CE.CDCENTCUST(+) AND E.CDCNAE = CN.CDCNAE(+) AND E.CDFPAS = FP.CDFPAS(+) AND E.CDNATUJURI = NA.CDNATUJURI(+) AND E.CDPAIS = P.CDPAIS(+) AND E.CDPAIS = S.CDPAIS(+) AND E.SGESTADO = S.SGESTADO(+) AND E.CDPAIS = M.CDPAIS(+) AND E.SGESTADO = M.SGESTADO(+) AND E.CDMUNICIPIO = M.CDMUNICIPIO(+) AND E.CDPAIS = B.CDPAIS(+) AND E.SGESTADO = B.SGESTADO(+) AND E.CDMUNICIPIO = B.CDMUNICIPIO(+) AND E.CDBAIRRO = B.CDBAIRRO(+) AND E.CDLOGRADOURO = L.CDLOGRADOURO(+) AND E.NRORG = L.NRORG(+) AND CE.NRORG(+) = E.NRORG AND CO.NRORG(+) = E.NRORG AND B.NRORG(+) = E.NRORG AND E.CDEMPRESA = :CDEMPRESA";
        
        $params = array(
            "CDEMPRESA"  => $cdempresa
        );
      
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetch();
    }
    
    public function getConfigUf($sguffavorecida, $cdreceitagnre) {
        $sql = "SELECT IDDETRECEITA,  IDPRODUTO, IDPERIODOREF, IDPERIODOAPUR,
                   IDDTVENCIMENTO, IDVRPRINCIPAL, IDVRTOTAL, IDDOCORIGEM,  IDDESTINATARIO,
                   IDCAMPOEXTRA,   IDEMITENTE,    IDPARCELA, IDCONVENIO,   IDDTPAGAMENTO
              FROM CONFIGUFGNRE
             WHERE SGUFFAVORECIDA = :P_SGUFFAVORECIDA 
                AND CDRECEITAGNRE =:P_CDRECEITAGNRE
                AND IDVERSAO = '2.0' ";
        
        $params = array(
            "P_SGUFFAVORECIDA"  => $sguffavorecida,
            "P_CDRECEITAGNRE"  => $cdreceitagnre,
        );
      
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetch();
    }
    
    public function adqCampoExtraObrig($sguffavorecida, $cdreceitagnre) {
        $sql = "SELECT MAX(NRCODIGO) AS NRCODIGO
                  FROM CAMPOEXTRAGNRE
                 WHERE SGUFFAVORECIDA = :P_SGUFFAVORECIDA
                   AND CDRECEITAGNRE  = :P_CDRECEITAGNRE
                   AND IDOBRIGCAMPO   = 'S'
                   AND IDVERSAO       = '2.0'";
        
        $params = array(
            "P_SGUFFAVORECIDA"  => $sguffavorecida,
            "P_CDRECEITAGNRE"  => $cdreceitagnre,
        );
      
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetch();
    }
    
    public function completaInfoGnreOnline($cdimposto, $cdempresa, $sgestado, $nrlanctonf, $cdfilial) {
        $sql = "SELECT
                     NVL(RP.CDTPITEMPROD,PG.CDPRODUTOGON) AS PRODUTO,
                     REPLACE(REPLACE(REPLACE(IE.NRINSJURIEST,'.'),'/'),'-') AS IDEMITENTE,
                     MODELO_DOCUMENTO_GNRE(NF.CDMODDOCFISC,'1','2.0') AS TPDOCUMENTO,
                     NF.NRNOTAFISC,
                     '' AS CONVENIO,
                     EP.NMRAZSOCEMPR   AS NMRAZSOCEMIT,
                     DECODE(IE.SGESTADO,NVL(EC.SGESTADO,EL.SGESTADO),IE.CDINSCESTA,ES.CDINSCESTAST) AS INSCESTAEMIT,
                     IE.DSENDEIEST     AS ENDEEMIT,
                     SUBSTR(MI.CDMUNICIBGE,3,5) AS MUNICIPIOEMIT,
                     IE.SGESTADO       AS SGESTADOEMIT,
                     REPLACE(REPLACE(IE.NRCEPIEST,'-'),'.') AS CEPEMIT,
                     IE.CDINSCESTA,
                     REPLACE(REPLACE(REPLACE(IE.NRTELEIEST,')'),'('),'-') AS TELEMIT,
                     DECODE(NVL(CL.IDTPIJURCLIE,FL.IDTPIJURFILI),'F','2','1') AS TPDESTINATARIO,
                     '1' AS TPEMITENTE,
                     REPLACE(REPLACE(REPLACE(NVL(CL.NRINSJURCLIE,FL.NRINSJURFILI),'.'),'/'),'-') AS IDDESTINATARIO,
                     NVL(CL.NRINSESTCLIE,FL.CDINSCESTA) AS INSCESTADEST,
                     NVL(CL.NMRAZSOCCLIE,FL.NMRAZSOCFILI) AS NMRAZSOCDEST,
                     SUBSTR(MC.CDMUNICIBGE,3,5) AS MUNICIPIODEST,
                     TO_CHAR(NF.DTEMISSAO,'MM') AS MES,
                     TO_CHAR(NF.DTEMISSAO,'YYYY') AS ANO,
                     DECODE(NVL(ES.IDPERIODOREFAPU,'I'),'I','M',ES.IDPERIODOREFAPU) AS IDPERIODOREFAPU,
                     TO_CHAR(NF.DTEMISSAO,'DD') AS DIA,
                     NF.NRACESSONFE    AS CHAVE,
                     'N' AS NFGERADA,
                     NF.CDFILIEMIT     AS FILIAL,
                     IE.NRINSJURIEST,
                     IE.IDTPIJURIEST,
                     'N' AS TIPOGUIA
                 FROM
                     NOTAFISCAL NF,
                     INSCRESTAD IE,
                     MUNICIPIO MI,
                     EMPRESA EP,
                     INSCRESTADST ES,
                     CLIENTE CL,
                     ENDECLIE EC,
                     FILIAL FL,
                     ENDEFILI EL,
                     MUNICIPIO MC,
                     PARAMGNRE PG,
                     (
                         SELECT
                             MAX(CDTPITEMPROD) AS CDTPITEMPROD,
                             IP.NRLANCTONF,
                             IP.CDFILIAL
                         FROM
                             ITEMPRODUTO IP,
                             RELACPRODITGEN RP,
                             ITEMPROIMPOS IT,
                             IMPOSTO IM
                         WHERE
                             IT.NRLANCTONF = :NRLANCTONF 
                             AND IT.CDFILIAL = :CDFILIAL 
                             AND IT.NRLANCTONF = IP.NRLANCTONF
                             AND IT.CDFILIAL = IP.CDFILIAL
                             AND IT.NRSEQITPROD = IP.NRSEQITPROD
                             AND IT.CDIMPOSTO = IM.CDIMPOSTO
                             AND IM.CDIMPOSTONFE =:P_CDIMPOSTO
                             AND IP.CDPRODUTO = RP.CDPRODUTO (+)
                         GROUP BY
                             IP.NRLANCTONF,
                             IP.CDFILIAL
                     ) RP
                 WHERE
                     NF.NRLANCTONF = :NRLANCTONF 
                     AND NF.CDFILIAL = :CDFILIAL
                     AND NF.CDEMPRESA = :CDEMPRESA
                     AND RP.NRLANCTONF (+) = NF.NRLANCTONF
                     AND RP.CDFILIAL (+) = NF.CDFILIAL
                     AND NF.CDEMPRESA = EP.CDEMPRESA
                     AND NF.CDEMPRESA = IE.CDEMPRESA
                     AND NF.CDINSCESTA = IE.CDINSCESTA
                     AND IE.CDPAIS = MI.CDPAIS
                     AND IE.SGESTADO = MI.SGESTADO
                     AND IE.CDMUNICIPIO = MI.CDMUNICIPIO
                     AND IE.CDEMPRESA = ES.CDEMPRESA (+)
                     AND IE.CDINSCESTA = ES.CDINSCESTA (+)
                     AND ES.SGESTADO (+) =:P_SGESTADO
                     AND NF.CDCLIENTE = CL.CDCLIENTE (+)
                     AND CL.CDCLIENTE = EC.CDCLIENTE (+)
                     AND EC.IDTPENDECLIE (+) = 'P'
                     AND DECODE(NF.IDENTRSAIDOP,'E',NF.CDFILIEMIT,NF.CDFILIRECE) = FL.CDFILIAL (+)
                     AND FL.CDFILIAL = EL.CDFILIAL (+)
                     AND EL.IDTPENDEFILI (+) = 'P'
                     AND NVL(EC.CDPAIS,EL.CDPAIS) = MC.CDPAIS
                     AND NVL(EC.SGESTADO,EL.SGESTADO) = MC.SGESTADO
                     AND NVL(EC.CDMUNICIPIO,EL.CDMUNICIPIO) = MC.CDMUNICIPIO
                     AND EP.CDEMPRESA = PG.CDEMPRESA (+)
                 UNION ALL 
                 SELECT
                     P.CDPRODUTOGON      AS PRODUTO,
                     REPLACE(REPLACE(REPLACE(IE.NRINSJURIEST,'.'),'/'),'-') AS IDEMITENTE,
                     '10' AS TPDOCUMENTO,
                     '' AS NRNOTAFISC,
                     '' AS CONVENIO,
                     EP.NMRAZSOCEMPR     AS NMRAZSOCEMIT,
                     NVL(ES.CDINSCESTAST,IE.CDINSCESTA) AS INSCESTAEMIT,
                     IE.DSENDEIEST       AS ENDEEMIT,
                     SUBSTR(MU.CDMUNICIBGE,3,5) AS MUNICIPIOEMIT,
                     IE.SGESTADO         AS SGESTADOEMIT,
                     REPLACE(REPLACE(IE.NRCEPIEST,'-'),'.') AS CEPEMIT,
                     IE.CDINSCESTA,
                     REPLACE(REPLACE(REPLACE(IE.NRTELEIEST,')'),'('),'-') AS TELEMIT,
                     '' AS TPDESTINATARIO,
                     '1' AS TPEMITENTE,
                     '' AS IDDESTINATARIO,
                     '' AS INSCESTADEST,
                     '' AS NMRAZSOCDEST,
                     '' AS MUNICIPIODEST,
                     TO_CHAR(MP.DTINIPERIODO,'MM') AS MES,
                     TO_CHAR(MP.DTINIPERIODO,'YYYY') AS ANO,
                     DECODE(NVL(ES.IDPERIODOREFAPU,'I'),'I','M',ES.IDPERIODOREFAPU) AS IDPERIODOREFAPU,
                     TO_CHAR(MP.DTINIPERIODO,'DD') AS DIA,
                     '' AS CHAVE,
                     'N' AS NFGERADA,
                     P.CDFILIAL          AS FILIAL,
                     IE.NRINSJURIEST,
                     IE.IDTPIJURIEST,
                     'M' AS TIPOGUIA
                 FROM
                     MAPAAPURACST MP,
                     EMPRESA EP,
                     INSCRESTAD IE,
                     INSCRESTADST ES,
                     MUNICIPIO MU,
                     PARAMGNRE P
                 WHERE
                     MP.CDEMPRESA = :CDEMPRESA
                     AND MP.CDEMPRESA = EP.CDEMPRESA
                     AND MP.CDEMPRESA = IE.CDEMPRESA
                     AND MP.CDINSCESTA = IE.CDINSCESTA
                     AND MP.CDEMPRESA = ES.CDEMPRESA (+)
                     AND MP.CDINSCESTA = ES.CDINSCESTA (+)
                     AND MP.CDINSCESTAST = ES.CDINSCESTAST (+)
                     AND IE.CDPAIS = MU.CDPAIS
                     AND IE.SGESTADO = MU.SGESTADO
                     AND IE.CDMUNICIPIO = MU.CDMUNICIPIO
                     AND EP.CDEMPRESA = P.CDEMPRESA (+)
                 UNION ALL
                 SELECT
                     P.CDPRODUTOGON      AS PRODUTO,
                     REPLACE(REPLACE(REPLACE(IE.NRINSJURIEST,'.'),'/'),'-') AS IDEMITENTE,
                     '10' AS TPDOCUMENTO,
                     '' AS NRNOTAFISC,
                     '' AS CONVENIO,
                     EP.NMRAZSOCEMPR     AS NMRAZSOCEMIT,
                     NVL(ES.CDINSCESTAST,IE.CDINSCESTA) AS INSCESTAEMIT,
                     IE.DSENDEIEST       AS ENDEEMIT,
                     SUBSTR(MU.CDMUNICIBGE,3,5) AS MUNICIPIOEMIT,
                     IE.SGESTADO         AS SGESTADOEMIT,
                     REPLACE(REPLACE(IE.NRCEPIEST,'-'),'.') AS CEPEMIT,
                     IE.CDINSCESTA,
                     REPLACE(REPLACE(REPLACE(IE.NRTELEIEST,')'),'('),'-') AS TELEMIT,
                     '' AS TPDESTINATARIO,
                     '1' AS TPEMITENTE,
                     '' AS IDDESTINATARIO,
                     '' AS INSCESTADEST,
                     '' AS NMRAZSOCDEST,
                     '' AS MUNICIPIODEST,
                     TO_CHAR(MA.DTINIPERIODO,'MM') AS MES,
                     TO_CHAR(MA.DTINIPERIODO,'YYYY') AS ANO,
                     DECODE(NVL(ES.IDPERIODOREFAPU,'I'),'I','M',ES.IDPERIODOREFAPU) AS IDPERIODOREFAPU,
                     TO_CHAR(MA.DTINIPERIODO,'DD') AS DIA,
                     '' AS CHAVE,
                     'N' AS NFGERADA,
                     P.CDFILIAL          AS FILIAL,
                     IE.NRINSJURIEST,
                     IE.IDTPIJURIEST,
                     'M' AS TIPOGUIA
                 FROM
                     GUIAAPURMAPANC GU,
                     EMPRESA EP,
                     INSCRESTAD IE,
                     INSCRESTADST ES,
                     MUNICIPIO MU,
                     PARAMGNRE P,
                     (
                         SELECT
                             MP.CDEMPRESA,
                             MP.CDINSCESTA,
                             MP.DTINIPERIODO,
                             ME.NRSEQUMAPAPU,
                             ME.NRSEQMAPAPURESTNC,
                             ME.SGESTADO
                         FROM
                             MAPAAPURAC MP,
                             MAPAAPURESTADONC ME
                         WHERE
                             ME.NRSEQUMAPAPU = MP.NRSEQUMAPAPU
                     ) MA
                 WHERE
                     GU.NRSEQUMAPAPU = MA.NRSEQUMAPAPU
                     AND GU.NRSEQMAPAPURESTNC = MA.NRSEQMAPAPURESTNC
                     AND MA.CDEMPRESA = :CDEMPRESA
                     AND MA.CDEMPRESA = EP.CDEMPRESA
                     AND MA.CDEMPRESA = IE.CDEMPRESA
                     AND MA.CDINSCESTA = IE.CDINSCESTA
                     AND MA.CDEMPRESA = ES.CDEMPRESA (+)
                     AND MA.CDINSCESTA = ES.CDINSCESTA (+)
                     AND MA.SGESTADO = ES.SGESTADO (+)
                     AND IE.CDPAIS = MU.CDPAIS
                     AND IE.SGESTADO = MU.SGESTADO
                     AND IE.CDMUNICIPIO = MU.CDMUNICIPIO
                     AND EP.CDEMPRESA = P.CDEMPRESA (+)";
        
        $params = array(
            "P_CDIMPOSTO"  => $cdimposto,
            "CDEMPRESA"  => $cdempresa,
            "P_SGESTADO" => $sgestado,
            "NRLANCTONF" => $nrlanctonf,
            "CDFILIAL" => $cdfilial,
        );
      
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetch();
    }
    
    public function getInscricaoEstadual($cdempresa, $cdinscesta) {
        $sql = "SELECT P.NMPAIS, S.NMESTADO, M.NMMUNICIPIO, M.CDMUNICIBGE, B.NMBAIRRO, L.DSLOGRADOURO, I.* FROM INSCRESTAD I, PAIS P, ESTADO S, MUNICIPIO M, BAIRRO B, LOGRADOURO L WHERE I.CDPAIS = P.CDPAIS(+) AND I.CDPAIS = S.CDPAIS(+) AND I.SGESTADO = S.SGESTADO(+) AND I.CDPAIS = M.CDPAIS(+) AND I.SGESTADO = M.SGESTADO(+) AND I.CDMUNICIPIO = M.CDMUNICIPIO(+) AND I.CDPAIS = B.CDPAIS(+) AND I.SGESTADO = B.SGESTADO(+) AND I.CDMUNICIPIO = B.CDMUNICIPIO(+) AND I.CDBAIRRO = B.CDBAIRRO(+) AND I.CDLOGRADOURO = L.CDLOGRADOURO(+) AND I.CDEMPRESA = :CDEMPRESA AND I.CDINSCESTA= :CDINSCESTA";
        
        $params = array(
            "CDEMPRESA"  => $cdempresa,
            "CDINSCESTA" => $cdinscesta
        );
      
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetch();
    }
    
    public function getNotaFiscal($cdfilial, $nrlanctonf) {
        $sql = "SELECT * FROM NOTAFISCAL WHERE CDFILIAL = :CDFILIAL AND NRLANCTONF = :NRLANCTONF";
        
        $params = array(
            "CDFILIAL"  => $cdfilial,
            "NRLANCTONF" => $nrlanctonf
        );
      
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetch();
    }
    
   
    
    #############################################################
    #                                                           #
    #           Funções  do Financeiro                          #
    #                                                           #
    #############################################################
    
    public function cancelaDadosFinanceiro($params) {
        $procedure = new StoredProcedure($this->entityManager->getConnection(), 'CANCELA_DADOS_FINANCEIRO ');

        $procedure->addParam(new Param('P_CDFILIAL', Param::PARAM_INPUT, $params["P_CDFILIAL"], Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_NRLANCTONF', Param::PARAM_INPUT, $params["P_NRLANCTONF"], Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_IDCANFIN', Param::PARAM_INPUT, $params["P_IDCANFIN"], Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_RETORNO', Param::PARAM_OUTPUT, null, Param::PARAM_TYPE_STR, 12));

        $retornoProcedure = $procedure->execute();
        $codigoRetorno = $retornoProcedure["P_RETORNO"];
        return $codigoRetorno;
    }

    
    /**
     * Busca Movimentação de Fluxo
     * @param $nrorg
     * @param $cdempresa
     * @param $cdfilial
     * @param $cdclassfina
     * @param $dtmovfluxo
     * @param $idmovfluxo
     * @return Movfluxo
     */
    public function buscaMovFluxo($nrorg, $cdempresa, $cdfilial, $cdclassfina, $dtmovfluxo, $idmovfluxo){
        $params = array();
        $params['nrorg'] = $nrorg;
        $params['cdempresa'] = $cdempresa;
        $params['cdfilial'] = $cdfilial;
        $params['cdclassfina'] = $cdclassfina;
        $params['dtmovfluxo'] = $dtmovfluxo;
        $params['idmovfluxo'] = $idmovfluxo;
        $movfluxo = $this->entityManager->getRepository(Movfluxo::class)->findOneBy($params);
        return $movfluxo;
    }
    
    /**
     * Atualiza Movimentação de Fluxo
     * @param Movfluxo $movfluxo
     * @param $vrmovfluxo
     * @param $idoperacao
     * @return Movfluxo
     */
    public function atualizaMovFluxo(Movfluxo $movfluxo, $vrmovfluxo, $idoperacao){
        // Verifica Operação Entrada/Saida
        if($idoperacao == 'E')
            $movfluxo->setVrentmfluxo($movfluxo->getVrentmfluxo() + $vrmovfluxo);
        else $movfluxo->setVrsaimfluxo($movfluxo->getVrsaimfluxo() + $vrmovfluxo);
        $movfluxo->setDtultatu(DateUtil::getDataAtual());

        return $movfluxo;
    }
    
    /**
     * Insere Movimentação de Fluxo
     * @param $nrorg
     * @param $cdempresa
     * @param $cdfilial
     * @param $cdclassfina
     * @param $dtmovfluxo
     * @param $idmovfluxo
     * @param $vrmovfluxo
     * @param $idoperacao
     * @return Movfluxo
     */
    public function insereMovFluxo($nrorg, $cdempresa, $cdfilial, $cdclassfina, $dtmovfluxo, $idmovfluxo, $vrmovfluxo, $idoperacao){
        /** @var Movfluxo $movfluxo */
        $movfluxo = new Movfluxo();
        $movfluxo->setNrorg($nrorg);
        $movfluxo->setCdfilial($cdfilial);
        $movfluxo->setCdempresa($cdempresa);
        $movfluxo->setCdclassfina($cdclassfina);
        $movfluxo->setDtmovfluxo($dtmovfluxo);
        $movfluxo->setIdmovfluxo($idmovfluxo);
        $movfluxo->setVrentmfluxo($idoperacao == 'E'?$vrmovfluxo:0);
        $movfluxo->setVrsaimfluxo($idoperacao == 'S'?$vrmovfluxo:0);
        $movfluxo->setDtinclusao(DateUtil::getDataAtual());
        $movfluxo->setDtultatu(DateUtil::getDataAtual());

        return $movfluxo;
    }
    
    public function insereNotaFiscal($nrorg, $cdfilial, $cdfilirece, $nrlanctonf, $cdempresa, $cdinscesta, $cdinscmuni, $cdforneced, $vrnotafisc, $nrnotafisc, $dtfimperiodo, $idtpijurnf, $nrinsjurnf, $cdtipooper){
        $notafiscal = new NotaFiscal();
        $notafiscal->setNrorg($nrorg);
        $notafiscal->setNrlanctonf($nrlanctonf);
        $notafiscal->setCdfilial($cdfilial);
        $notafiscal->setCdempresa($cdempresa);
        $notafiscal->setCdinscesta($cdinscesta);
        $notafiscal->setCdinscmuni($cdinscmuni);
        $notafiscal->setCdforneced($cdforneced);
        $notafiscal->setVrnotafisc($vrnotafisc);
        $notafiscal->setNrnotafisc($nrnotafisc);
        $notafiscal->setDtemissao($dtfimperiodo);
        $notafiscal->setDtentrsaid($dtfimperiodo);
        $notafiscal->setDtlancamennf($dtfimperiodo);
        $notafiscal->setIdtpijurnf($idtpijurnf);
        $notafiscal->setNrinsjurnf($nrinsjurnf);
        $notafiscal->setCdtipooper($cdtipooper);
        $notafiscal->setIdbloqueado('S');
        $notafiscal->setVrprodutos($vrnotafisc);
        $notafiscal->setIdentrsaidop('E');
        $notafiscal->setCdfiliRece($cdfilirece);
        $notafiscal->setIdsistema('FIS');
        $notafiscal->setIdstatus('A');
        
        $notafiscal->setDtinclusao(DateUtil::getDataAtual());
        $notafiscal->setDtultatu(DateUtil::getDataAtual());
        
        $this->entityManager->persist($notafiscal);
    }

    public function insereBoleto($nrorg, $cdfilial, $nrlanbolpag, $cdforneced, $cdempresa, $vrbolpag, $dtvenbolpag, $cdtpbaixapag, $cdreceitaguia, $dtapuracguia){
        $boletpagar = new Boletpagar();
        $boletpagar->setNrorg($nrorg);
        $boletpagar->setCdfilial($cdfilial);
        $boletpagar->setNrlanbolpag($nrlanbolpag);
        $boletpagar->setCdforneced($cdforneced);
        $boletpagar->setCdempresa($cdempresa);
        $boletpagar->setVrbolpag($vrbolpag);
        $boletpagar->setDtvenbolpag($dtvenbolpag);
        $boletpagar->setCdtpbaixapag($cdtpbaixapag);
        $boletpagar->setVrfinalbpag($vrbolpag);
        $boletpagar->setCdreceitaguia($cdreceitaguia);
        $boletpagar->setDtapuracguia(DateUtil::getDataDeString(substr($dtapuracguia, 0, 10)));
        
        $boletpagar->setDtinclusao(DateUtil::getDataAtual());
        $boletpagar->setDtultatu(DateUtil::getDataAtual());
        
        $this->entityManager->persist($boletpagar);
    }
    
    public function insereContaPagar($nrorg, $cdfilial, $nrlanctonf, $dtorigvenpag, $vrorigpag, $dscontapag, $nrlanbolpag, $cdempresa, $cdclassfina, $cdtpctpagar){
        $contapagar = new Contapagar();
        $contapagar->setNrorg($nrorg);
        $contapagar->setCdfilial($cdfilial);
        $contapagar->setNrlanctonf($nrlanctonf);
        $contapagar->setNrseqvenpag('01');
        $contapagar->setDtorigvenpag($dtorigvenpag);
        $contapagar->setVrorigpag($vrorigpag);
        $contapagar->setDscontapag($dscontapag);
        $contapagar->setCdfilibpag($cdfilial);
        $contapagar->setNrlanbolpag($nrlanbolpag);
        $contapagar->setCdempresa($cdempresa);
        $contapagar->setCdclassfina($cdclassfina);
        $contapagar->setCdtpctpagar($cdtpctpagar);
        $contapagar->setNrparcelapag('00');
        $contapagar->setQtdiasvenpag(0);
        $contapagar->setIdstatpag('A');
        $contapagar->setIdmovfluxo('E');
        $contapagar->setVratuapag($vrorigpag);
        $contapagar->setDtatuavenpag($dtorigvenpag);
        
    

        $contapagar->setDtinclusao(DateUtil::getDataAtual());
        $contapagar->setDtultatu(DateUtil::getDataAtual());
        
        $this->entityManager->persist($contapagar);
    }
    
    public function insereRatctpagar($nrorg, $cdfilial, $cdfiratctpag, $nrlanctonf, $vrparcela, $cdcentcusnf, $nrseqvenpag){
        $ratctpagar = new Ratctpagar();
        $ratctpagar->setNrorg($nrorg);
        $ratctpagar->setCdfilial($cdfilial);
        $ratctpagar->setNrlanctonf($nrlanctonf);
        $ratctpagar->setNrparcelapag('00');
        $ratctpagar->setVrratccctpag($vrparcela);
        $ratctpagar->setCdcentcust($cdcentcusnf);
        $ratctpagar->setCdfiratctpag($cdfiratctpag);
        $ratctpagar->setNrseqvenpag($nrseqvenpag);
        
        $ratctpagar->setDtinclusao(DateUtil::getDataAtual());
        $ratctpagar->setDtultatu(DateUtil::getDataAtual());
        
        $this->entityManager->persist($ratctpagar);

     }
    
    public function deleteContaPagar($params) {
        $sql = "DELETE FROM CONTAPAGAR WHERE CDFILIAL =:CDFILIAL AND NRLANCTONF = :NRLANCTONF";

        $std = $this->connection->prepare($sql);
        $std->execute($params);
    } 
    
    public function deleteNotaFiscal($params) {
        $sql = "DELETE FROM RELACMAPANF WHERE CDFILIAL =:CDFILIAL AND NRLANCTONF = :NRLANCTONF";
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        
        $sql = "DELETE FROM NOTAFISCAL  WHERE CDFILIAL =:CDFILIAL AND NRLANCTONF = :NRLANCTONF";
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    } 
    
    
    
    #############################################################
    #                                                           #
    #           Funções  de Lote (RJ)                           #
    #                                                           #
    #############################################################
       
    public function insereLotegnre($nrseqlote, $cdempresa, $cdinscesta, $sgestado, $idstatuslote, $nrrecibo, $dthrrecibo, $nrorg){
       
        $sql = " INSERT INTO LOTEGNRE (
                    NRSEQLOTE, CDEMPRESA, CDINSCESTA, SGESTADO, IDSTATUSLOTE, NRRECIBO, DTHRRECIBO, NRORG
                )
                VALUES( 
                    :NRSEQLOTE, :CDEMPRESA, :CDINSCESTA, :SGESTADO, :IDSTATUSLOTE, :NRRECIBO, :DTHRRECIBO, :NRORG)";
                    
        $params = array(
            "NRSEQLOTE"    => $nrseqlote,
            "CDEMPRESA"    => $cdempresa,
            "CDINSCESTA"   => $cdinscesta,
            "SGESTADO"     => $sgestado,
            "IDSTATUSLOTE" => $idstatuslote,
            "NRRECIBO"     => $nrrecibo,
            "DTHRRECIBO"   => $dthrrecibo,
            "NRORG"        => $nrorg
        );
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
    
    public function updateTransmissaoDocarrecadpag($nrseqdocarrecad, $idtransmissao, $nrseqlote) {
        $params = array(
            "NRSEQDOCARRECAD" => $nrseqdocarrecad,
            "IDTRANSMISSAO"   => $idtransmissao,
            "NRSEQLOTE"       => $nrseqlote
           
        );

        $sql = " UPDATE DOCARRECADPAG SET IDTRANSMISSAO =  :IDTRANSMISSAO, NRSEQLOTE = :NRSEQLOTE WHERE NRSEQDOCARRECAD =:NRSEQDOCARRECAD";
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
    
    public function deleteTranmissaoLote() {
        $sql = " DELETE
                   FROM ERROSLOTEGNRE
                  WHERE NRSEQLOTE NOT IN
                      (SELECT NRSEQLOTE FROM GNRENF WHERE NRSEQLOTE IS NOT NULL
                      UNION ALL
                      SELECT NRSEQLOTE FROM GNREMAPA WHERE NRSEQLOTE IS NOT NULL
                      UNION ALL
                      SELECT NRSEQLOTE FROM DOCARRECADPAG WHERE NRSEQLOTE IS NOT NULL
                      )";
        $std = $this->connection->prepare($sql);
        $std->execute();
        
        
        $sql = " DELETE
                   FROM LOTEGNRE
                  WHERE NRSEQLOTE NOT IN
                      (SELECT NRSEQLOTE FROM GNRENF WHERE NRSEQLOTE IS NOT NULL
                      UNION ALL
                      SELECT NRSEQLOTE FROM GNREMAPA WHERE NRSEQLOTE IS NOT NULL
                      UNION ALL
                      SELECT NRSEQLOTE FROM DOCARRECADPAG WHERE NRSEQLOTE IS NOT NULL
                      )";
        $std = $this->connection->prepare($sql);
        $std->execute();
    }
    
    public function getLotegnre ($nrseqlote){
        $sql = " SELECT *
                   FROM LOTEGNRE
                  WHERE NRSEQLOTE = :NRSEQLOTE";
       
        $params = array(
            "NRSEQLOTE"  => $nrseqlote
        );
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetch();
       
    }

    public function updateConsultaLoteDocarrecadpag($nrseqdocarrecad, $idtransmissao, $nrnossonumero, $nrcodigobarra, $nrLinhaDigitavel, $vrmulta, $vrjuros, $dtlimitepag, $vrICMSPrincipal) {
        $params = array(
            "NRSEQDOCARRECAD" => $nrseqdocarrecad,
            "IDTRANSMISSAO"   => $idtransmissao,
            "NRNOSSONUMERO"   => $nrnossonumero,
            "NRCODIGOBARRA"   => $nrcodigobarra,
            "NRLINHADIGI"     => $nrLinhaDigitavel,
            "VRMULTA"         => $vrmulta,
            "VRJUROS"         => $vrjuros,
            "VRPRINCIPAL"     => $vrICMSPrincipal,
            "DTLIMITEPAG"     => $dtlimitepag
            );
        $sql = " UPDATE DOCARRECADPAG 
                    SET IDTRANSMISSAO = :IDTRANSMISSAO, 
                        NRNOSSONUMERO = :NRNOSSONUMERO,
                        NRCODIGOBARRA = :NRCODIGOBARRA ,
                        NRLINHADIGI   = :NRLINHADIGI ,
                        VRMULTA       = :VRMULTA ,
                        VRJUROS       = :VRJUROS ,
                        VRPRINCIPAL   = :VRPRINCIPAL,
                        DTLIMITEPAG   = :DTLIMITEPAG 
                  WHERE NRSEQDOCARRECAD =:NRSEQDOCARRECAD";
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
    
    public function updateConsultaLoteDocarrecadpagError($nrseqdocarrecad, $idtransmissao) {
        $params = array(
            "NRSEQDOCARRECAD" => $nrseqdocarrecad,
            "IDTRANSMISSAO"   => $idtransmissao,
            );
        $sql = " UPDATE DOCARRECADPAG 
                    SET IDTRANSMISSAO = :IDTRANSMISSAO
                  WHERE NRSEQDOCARRECAD =:NRSEQDOCARRECAD";
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
    
    public function updateLoteGnre($nrseqlote, $idstatuslote) {
        $params = array(
            "NRSEQLOTE"   => $nrseqlote,
            "IDSTATUSLOTE" => $idstatuslote
        );

        $sql = " UPDATE LOTEGNRE 
                    SET IDSTATUSLOTE = :IDSTATUSLOTE
                  WHERE NRSEQLOTE =:NRSEQLOTE";
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
    
    public function insertErroLoteGnre($nrseqlote, $cderroitem, $dserroitem, $cdidentdeb, $nrorg) {
        $params = array(
            "NRSEQLOTE"  => $nrseqlote,
            "CDERROITEM" => $cderroitem,
            "DSERROITEM" => $dserroitem,
            "CDIDENTDEB" => $cdidentdeb,
            "NRORG"      => $nrorg
        );

        $sql = "INSERT INTO 
                        ERROSLOTEGNRE (
                            NRSEQLOTE, 
                            CDERROITEM, 
                            DSERROITEM, 
                            CDIDENTDEB,
                            NRORG
                        ) VALUES (
                            :NRSEQLOTE, 
                            :CDERROITEM, 
                            :DSERROITEM, 
                            :CDIDENTDEB,
                            :NRORG
                            )";
                            
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
    
    public function getDocumentoLote ($nrseqlote){
        $sql = "SELECT *
                   FROM DOCARRECADPAG
                  WHERE NRSEQLOTE = :NRSEQLOTE";
       
        $params = array(
            "NRSEQLOTE"  => $nrseqlote
        );
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
       
    }
    
    public function getDocumentoPorId ($nrseqdocarrecad){
        $sql = "SELECT *
                   FROM DOCARRECADPAG
                  WHERE NRSEQDOCARRECAD = :NRSEQDOCARRECAD";
       
        $params = array(
            "NRSEQDOCARRECAD"  => $nrseqdocarrecad
        );
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetch();
       
    }
    
    #############################################################
    #                                                           #
    #           Funções de guias                                #
    #                                                           #
    #############################################################
    
    public function getDadosGuiaGnreTxt( $nrseqdocarrecad, $nrorg ) {
        $sql    = "SELECT
                      /*DAP.IDTIPOGUIA, DAP.NRSEQDOCARRECAD, DAP.NRSEQUMAPAPU,*/
                      DAP.CDRECEITA    CDRECEITA,
                      E.NMRAZSOCEMPR   NMCONTRIBUINTE,
                      IE.DSENDEIEST    ENDERECO,
                      M.NMMUNICIPIO    MUNICIPIO,
                      IE.SGESTADO      UF,
                      IE.NRTELEIEST    TELEFONE,
                      IE.NRCEPIEST     CEP,
                      IE.NRINSJURIEST  CNPJ,
                      IE.CDINSCESTA    INSCESTADUAL,
                      DAP.DSINFOCOMPL  OBSERVACOES,
                      ''               CONVENIO,
                      TO_CHAR( DAP.DTVENCIMENTO, 'DD/MM/YYYY' ) DTVENCIMENTO,
                      NF.NRNOTAFISC    NRDOCUMENTO,
                      TO_CHAR( DECODE( NF.IDENTRSAIDOP, 'E', NF.DTENTRSAID, 'S', NF.DTEMISSAO ), 'DD/YYYY') REFERENCIA,
                      TRIM( REPLACE( TO_CHAR( NVL( DAP.VRPRINCIPAL, 0 ), '999999990.99'), '.', ',' ) )        VRRECEITA,
                      '0,00'                                                                                ATUMONETARIA,
                      TRIM( REPLACE( TO_CHAR( NVL( DAP.VRJUROS,     0 ), '999999990.99'), '.', ',' ) )        VRJUROS,
                      TRIM( REPLACE( TO_CHAR( NVL( DAP.VRMULTA,     0 ), '999999990.99'), '.', ',' ) )        VRMULTA
                    FROM DOCARRECADPAG DAP
                    JOIN NOTAFISCAL NF ON NF.NRLANCTONF = DAP.NRLANCTONF
                                      AND NF.NRORG      = DAP.NRORG
                    JOIN EMPRESA E ON E.NRORG     = DAP.NRORG
                                  AND E.CDEMPRESA = NF.CDEMPRESA
                    JOIN INSCRESTAD IE ON IE.NRORG      = DAP.NRORG
                                      AND IE.CDEMPRESA  = NF.CDEMPRESA
                                      AND IE.CDINSCESTA = NF.CDINSCESTA
                    JOIN MUNICIPIO M ON M.NRORG       = DAP.NRORG
                                    AND M.CDPAIS      = IE.CDPAIS
                                    AND M.SGESTADO    = IE.SGESTADO
                                    AND M.CDMUNICIPIO = IE.CDMUNICIPIO
                    WHERE DAP.NRORG           = :NRORG
                      AND DAP.NRSEQDOCARRECAD = :NRSEQDOCARRECAD";
                        
        $params = array(
            'NRSEQDOCARRECAD' => $nrseqdocarrecad,
            'NRORG' => $nrorg );
        $std = $this->connection->prepare( $sql );
        $std->execute( $params );
        return $std->fetch();
    }
    
}